unit fmuMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, DrvFRLib_TLB;

type
  TfmMain = class(TForm)
    btnCreateCheck: TButton;
    lbl1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCreateCheckClick(Sender: TObject);
  private
    FDrv: TDrvFR;
    procedure CreateCheck;
    function GetQuantityTLVData(AMeasureUnit: Integer; AQuantity: double): string;
    function GetDivisionalQuantityTLVData(ANumerator: Integer; ADenominator: Integer): string;
    procedure Check(ACode: Integer; const AMsg: string = '');
    procedure BeginSTLVTag(ATag: Integer);
    procedure AddTagByte(ATag: Integer; const AValue: Byte);
    procedure AddTagStr(ATag: Integer; const AValue: string);
    procedure AddTagUnixTime(ATag: Integer; const AValue: TDateTime);
    procedure SendSTLVTag(ATag: Integer);
    procedure SendTagStrOperation(ATag: Integer; const AValue: string);
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.dfm}

const
  //    HEX 
  KM1_HEX = '3031303132333435363738393031323332313E4C522C4D472A2575304B32687A1D' +
            '3931303035381D393236462F6F784F6365505468436E55504434486D47454B3146' +
            '4753764C4237743662596B636636436A617A654F4238762B51335863576C597043' +
            '6A464E47447334335351654F766B797A5455506E302F476431714957773D3D';

  //  EAN13  HEX 
  CODE_EAN13_HEX = '32343030303030303332363332';

  KM2_HEX = '30313031323334353637383930313233323128446D79675938362B634B3E4C451D' +
            '3931303035381D39327A336951717462655A6474704B497A3571724F666D334F31' +
            '6C717352682F74457A425A47684C7A6E762F4367727A7249476E6E484936643165' +
            '5A5259432F4C7470684B6643595A48554E6D443562706F6E3672494F773D3D';

  KM3_HEX = '3031303132333435363738393031323332314468674A577255526F35576C4C721D' +
            '3931303035381D393238543231777350397543416C547A2F376C6C6A374D426150' +
            '474939573543544C5332564E594D655642596D7555764E36347054545038472B65' +
            '736D566653657231376A535855343068416646717558644C69646B53673D3D';

//  
procedure TfmMain.CreateCheck;
begin
  //  
  FDrv.ConnectionType := 6; // TCP
  FDrv.ProtocolType := 0;
  FDrv.UseIPAddress := True;
  FDrv.IPAddress := '192.168.137.111';
  FDrv.TCPPort := 7778;
  FDrv.Timeout := 10000;

  //  
  //    
  Check(FDrv.GetECRStatus, 'GetECRStatus');
  if FDrv.ECRMode = 4 then
    Check(FDrv.OpenSession, 'OpenSession')
  else
    if FDrv.ECRMode = 8 then //    
      Check(FDrv.SysAdminCancelCheck, 'SysAdminCancelCheck');

  // ,      
  //   
  //   
  FDrv.BarcodeHex := KM1_HEX;
  FDrv.ItemStatus := 1;   //   -  , 
  FDrv.TLVDataHEX := '';
  Check(FDrv.FNCheckItemBarcode, 'FNCheckItemBarcode');
  //      ,    .
  //   ,   FNAcceptMarkingCode,   
  //  FNDeclineMarkingCode       
  Check(FDrv.FNAcceptMarkingCode, 'FNAcceptMarkingCode');  // ,     

  //   
  FDrv.BarcodeHex := KM2_HEX;
  FDrv.ItemStatus := 2;   //   -     
  FDrv.TLVDataHex := GetQuantityTLVData(0, 100);  //    
  Check(FDrv.FNCheckItemBarcode, 'FNCheckItemBarcode'); //   
  //...
  Check(FDrv.FNAcceptMarkingCode, 'FNAcceptMarkingCode');  // ,     

  //   
  FDrv.BarcodeHex := KM3_HEX;
  FDrv.ItemStatus := 2;   //   -     
  FDrv.TLVDataHex := GetQuantityTLVData(0, 1);  //    
  FDrv.TLVDataHex := FDrv.TLVDataHex + GetDivisionalQuantityTLVData(1, 2);  //  ,   
  Check(FDrv.FNCheckItemBarcode, 'FNCheckItemBarcode'); //   
  //...
  Check(FDrv.FNAcceptMarkingCode, 'FNAcceptMarkingCode');  // ,     

  //  

  //  
  FDrv.CheckType := 0;
  Check(FDrv.OpenCheck, 'OpenCheck');  //  

  //   
  //    
  BeginSTLVTag(1256);
  AddTagStr(1227, '  '); // 
  AddTagStr(1228, '526317984689'); //  
  AddTagstr(1243, FormatDateTime('dd.mm.yyyy', EncodeDate(1980, 1, 1))); // .. 
  AddTagStr(1244, '643'); //  
  AddTagStr(1245, '21'); //   . 
  AddTagStr(1246, '1234 123456'); //  
  AddTagStr(1254, ''); //  
  SendSTLVTag(1256);

  //    
  BeginSTLVTag(1270);
  AddTagByte(1271, 0); // 
  AddTagStr(1272, ''); //  
  AddTagUnixTime(1273, Now); // ,  
  SendSTLVTag(1270);

  //   
  BeginSTLVTag(1261);
  AddTagStr(1262, '001'); //  
  AddTagstr(1263, FormatDateTime('dd.mm.yyyy', Now)); //  - 
  AddTagStr(1264, '123'); //  - 
  AddTagStr(1265, ''); //  
  SendSTLVTag(1261);

  //   
  FDrv.StringForPrinting := ' 1';
  FDrv.CheckType := 1;
  FDrv.Quantity := 1;
  FDrv.Price := 100;
  FDrv.Tax1 := 1;
  FDrv.Department := 1;
  FDrv.TaxValueEnabled := False;
  FDrv.Summ1Enabled := False;
  FDrv.PaymentTypeSign := 4;
  FDrv.PaymentItemSign := 1;
  FDrv.MeasureUnit := 0; //  
  FDrv.DivisionalQuantity := False; //   
  Check(FDrv.FNOperation, 'FNOperation'); //  
  FDrv.BarcodeHex := KM1_HEX;
  Check(FDrv.FNSendItemBarcode); //    
  FDrv.BarcodeHex := CODE_EAN13_HEX; //      EAN13, 
  Check(FDrv.FNSendItemBarcode, 'FNSendItemBarcode');

  //   
  FDrv.StringForPrinting := ' 2';
  FDrv.Quantity := 100;
  FDrv.Price := 10;
  FDrv.Tax1 := 1;
  FDrv.Department := 1;
  FDrv.TaxValueEnabled := False;
  FDrv.Summ1Enabled := False;
  FDrv.PaymentTypeSign := 4;
  FDrv.PaymentItemSign := 1;
  FDrv.MeasureUnit := 0; //  
  FDrv.DivisionalQuantity := False; //   
  Check(FDrv.FNOperation, 'FNOperation');
  FDrv.BarcodeHex := KM2_HEX;
  Check(FDrv.FNSendItemBarcode, 'FNSendItemBarcode'); //    

  //   
  FDrv.StringForPrinting := ' 3';
  FDrv.Quantity := 1;
  FDrv.Price := 10;
  FDrv.Tax1 := 1;
  FDrv.Department := 1;
  FDrv.TaxValueEnabled := False;
  FDrv.Summ1Enabled := False;
  FDrv.PaymentTypeSign := 4;
  FDrv.PaymentItemSign := 1;
  FDrv.MeasureUnit := 0; //  
  FDrv.DivisionalQuantity := True; //    
  FDrv.Numerator := '1';  // 
  FDrv.Denominator := '2'; // 
  Check(FDrv.FNOperation, 'FNOperation');
  FDrv.BarcodeHex := KM3_HEX;
  Check(FDrv.FNSendItemBarcode, 'FNSendItemBarcode'); //    

  //   ,    
  SendTagStrOperation(1262, '001'); //  
  SendTagStrOperation(1263, FormatDateTime('dd.mm.yyyy', Now)); //  - 
  SendTagStrOperation(1264, '123'); //  - 
  SendTagStrOperation(1265, ''); //  

  // 
  FDrv.Summ1 := 10000;
  FDrv.Summ2 := 0;
  FDrv.Summ3 := 0;
  FDrv.Summ4 := 0;
  FDrv.Summ5 := 0;
  FDrv.Summ6 := 0;
  FDrv.Summ7 := 0;
  FDrv.Summ8 := 0;
  FDrv.Summ9 := 0;
  FDrv.Summ10 := 0;
  FDrv.Summ11 := 0;
  FDrv.Summ12 := 0;
  FDrv.Summ13 := 0;
  FDrv.Summ14 := 0;
  FDrv.Summ15 := 0;
  FDrv.Summ16 := 0;
  FDrv.TaxType := 1;
  FDrv.RoundingSumm := 0;
  FDrv.StringForPrinting := '';
  Check(FDrv.FNCloseCheckEx, 'FNCloseCheckEx');

  FDrv.Disconnect;
end;


procedure TfmMain.BeginSTLVTag(ATag: Integer);
begin
  FDrv.TagNumber := ATag;
  Check(FDrv.FNBeginSTLVTag);
end;

procedure TfmMain.btnCreateCheckClick(Sender: TObject);
begin
  CreateCheck;
end;

procedure TfmMain.AddTagByte(ATag: Integer; const AValue: Byte);
begin
  FDrv.TagNumber := ATag;
  FDrv.TagType := ttByte;
  FDrv.TagValueInt := AValue;
  Check(FDrv.FNAddTag);
end;

procedure TfmMain.AddTagStr(ATag: Integer; const AValue: string);
begin
  FDrv.TagNumber := ATag;
  FDrv.TagType := ttString;
  FDrv.TagValueStr := AValue;
  Check(FDrv.FNAddTag);
end;

procedure TfmMain.AddTagUnixTime(ATag: Integer; const AValue: TDateTime);
begin
  FDrv.TagNumber := ATag;
  FDrv.TagType := ttUnixTime;
  FDrv.TagValueDateTime := AValue;
  Check(FDrv.FNAddTag);
end;

procedure TfmMain.SendSTLVTag(ATag: Integer);
begin
  FDrv.TagNumber := ATag;
  Check(FDrv.FNSendSTLVTag, ' ' + ATag.ToString);
end;

procedure TfmMain.SendTagStrOperation(ATag: Integer; const AValue: string);
begin
  FDrv.TagNumber := ATag;
  FDrv.TagType := ttString;
  FDrv.TagValueStr := AValue;
  Check(FDrv.FNSendTagOperation, ' ' + ATag.ToString);
end;

function TfmMain.GetQuantityTLVData(AMeasureUnit: Integer; AQuantity: double): string;
begin
  FDrv.TagNumber := 2108;
  FDrv.TagType := ttByte;
  FDrv.TagValueInt := AMeasureUnit;
  Check(FDrv.GetTagAsTLV);
  Result := FDrv.TLVDataHEX;

  FDrv.TagNumber := 1023;
  FDrv.TagType := ttFVLND;
  FDrv.TagValueFVLND := AQuantity;
  Check(FDrv.GetTagAsTLV);
  Result := Result + FDrv.TLVDataHex;
end;

function TfmMain.GetDivisionalQuantityTLVData(ANumerator: Integer; ADenominator: Integer): string;
begin
  FDrv.TagNumber := 1291;
  FDrv.TagType := ttSTLV;
  FDrv.FNBeginSTLVTag;
  FDrv.TagNumber := 1293;
  FDrv.TagType := ttVLN;
  FDrv.TagValueVLN := ANumerator.ToString;
  FDrv.FNAddTag;
  FDrv.TagNumber := 1294;
  FDrv.TagType := ttVLN;
  FDrv.TagValueVLN := ADenominator.ToString;
  FDrv.FNAddTag;
  FDrv.TagNumber := 1291;
  FDrv.TagType := ttSTLV;
  Check(FDrv.GetTagAsTLV);
  Result := FDrv.TLVDataHex;
end;

procedure TfmMain.Check(ACode: Integer; const AMsg: string);
var
  ErrorCode: string;
begin
  if ACode <> 0 then
  begin
    ErrorCode := FDrv.ResultCodeDescription;
    FDrv.Disconnect;
    raise Exception.CreateFmt(' (%s): %d, %s',[AMsg, ACode, ErrorCode]);
  end;
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  FDrv := TDrvFR.Create(nil);
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
  FDrv.Free;
end;

end.
